@@ -21,15 +21,7 @@
 # pylint: disable=broad-exception-caught
 # pylint: disable=wrong-import-position
 
-"""
-Utilities for obtaining information about accessible hypertext and hyperlinks
-These utilities are app-type- and toolkit-agnostic. Utilities that might have
-different implementations or results depending on the type of app (e.g. terminal,
-chat, web) or toolkit (e.g. Qt, Gtk) should be in script_utilities.py file(s).
-
-N.B. There are currently utilities that should never have custom implementations
-that live in script_utilities.py files. These will be moved over time.
-"""
+"""Utilities for obtaining information about accessible hypertext and hyperlinks."""
 
 __id__        = "$Id$"
 __version__   = "$Revision$"
@@ -39,6 +31,7 @@ __license__   = "LGPL"
 
 import os
 import re
+from typing import Optional
 from urllib.parse import urlparse
 
 import gi
@@ -52,7 +45,7 @@ class AXHypertext:
     """Utilities for obtaining information about accessible hypertext and hyperlinks."""
 
     @staticmethod
-    def _get_link_count(obj):
+    def _get_link_count(obj: Atspi.Accessible) -> int:
         """Returns the number of hyperlinks in obj."""
 
         if not AXObject.supports_hypertext(obj):
@@ -62,15 +55,15 @@ class AXHypertext:
             count = Atspi.Hypertext.get_n_links(obj)
         except Exception as error:
             msg = f"AXHypertext: Exception in _get_link_count: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return 0
 
         tokens = ["AXHypertext:", obj, f"reports {count} hyperlinks"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return count
 
     @staticmethod
-    def _get_link_at_index(obj, index):
+    def _get_link_at_index(obj: Atspi.Accessible, index: int) -> Optional[Atspi.Hyperlink]:
         """Returns the hyperlink object at the specified index."""
 
         if not AXObject.supports_hypertext(obj):
@@ -80,13 +73,15 @@ class AXHypertext:
             link = Atspi.Hypertext.get_link(obj, index)
         except Exception as error:
             msg = f"AXHypertext: Exception in _get_link_at_index: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return None
 
         return link
 
     @staticmethod
-    def get_all_links_in_range(obj, start_offset, end_offset):
+    def get_all_links_in_range(
+        obj: Atspi.Accessible, start_offset: int, end_offset: int
+    ) -> list[Atspi.Hyperlink]:
         """Returns all the hyperlinks in obj who started within the specified range."""
 
         links = []
@@ -98,11 +93,11 @@ class AXHypertext:
 
         tokens = [f"AXHypertext: {len(links)} hyperlinks found in", obj,
                   f"between start: {start_offset} and end: {end_offset}"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return links
 
     @staticmethod
-    def get_all_links(obj):
+    def get_all_links(obj: Atspi.Accessible) -> list[Atspi.Hyperlink]:
         """Returns a list of all the hyperlinks in obj."""
 
         links = []
@@ -112,11 +107,11 @@ class AXHypertext:
                 links.append(link)
 
         tokens = [f"AXHypertext: {len(links)} hyperlinks found in", obj]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return links
 
     @staticmethod
-    def get_link_uri(obj, index=0):
+    def get_link_uri(obj: Atspi.Accessible, index: int = 0) -> str:
         """Returns the URI associated with obj at the specified index."""
 
         try:
@@ -124,65 +119,69 @@ class AXHypertext:
             uri = Atspi.Hyperlink.get_uri(link, index)
         except Exception as error:
             msg = f"AXHypertext: Exception in get_link_uri: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return ""
 
         tokens = ["AXHypertext: URI of", obj, f"at index {index} is {uri}"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return uri
 
     @staticmethod
-    def get_link_start_offset(obj):
+    def get_link_start_offset(obj: Atspi.Accessible) -> int:
         """Returns the start offset of obj in the associated text."""
 
         if isinstance(obj, Atspi.Hyperlink):
             link = obj
+            obj = Atspi.Hyperlink.get_object(link, 0)
         else:
             link = Atspi.Accessible.get_hyperlink(obj)
 
         if link is None:
             tokens = ["AXHypertext: Couldn't get hyperlink for", obj]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return -1
 
         try:
             offset = Atspi.Hyperlink.get_start_index(link)
         except Exception as error:
             msg = f"AXHypertext: Exception in get_link_start_offset: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return -1
 
         tokens = ["AXHypertext: Start offset of", obj, f"is {offset}"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return offset
 
     @staticmethod
-    def get_link_end_offset(obj):
+    def get_link_end_offset(obj: Atspi.Accessible) -> int:
         """Returns the end offset of obj in the associated text."""
 
         if isinstance(obj, Atspi.Hyperlink):
             link = obj
+            obj = Atspi.Hyperlink.get_object(link, 0)
         else:
             link = Atspi.Accessible.get_hyperlink(obj)
 
         if link is None:
             tokens = ["AXHypertext: Couldn't get hyperlink for", obj]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return -1
 
         try:
             offset = Atspi.Hyperlink.get_end_index(link)
         except Exception as error:
             msg = f"AXHypertext: Exception in get_link_end_offset: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return -1
 
         tokens = ["AXHypertext: End offset of", obj, f"is {offset}"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return offset
 
     @staticmethod
-    def get_link_basename(obj, index=0, remove_extension=False):
+    def get_link_basename(
+        obj: Atspi.Accessible, index: int = 0, remove_extension: bool = False
+    ) -> str:
         """Strip directory and suffix off of the URL associated with obj."""
 
         uri = AXHypertext.get_link_uri(obj, index)
@@ -196,11 +195,11 @@ class AXHypertext:
             basename = re.sub(r"[-_]", " ", basename)
 
         tokens = ["AXHypertext: Basename for link", obj, f"is '{basename}'"]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return basename
 
     @staticmethod
-    def get_child_at_offset(obj, offset):
+    def get_child_at_offset(obj: Atspi.Accessible, offset: int) -> Optional[Atspi.Accessible]:
         """Returns the embedded-object child of obj at the specified offset."""
 
         if not AXObject.supports_hypertext(obj):
@@ -210,7 +209,7 @@ class AXHypertext:
             index = Atspi.Hypertext.get_link_index(obj, offset)
         except Exception as error:
             msg = f"AXHypertext: Exception in get_child_at_offset: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return None
 
         if index < 0:
@@ -224,15 +223,15 @@ class AXHypertext:
             child = Atspi.Hyperlink.get_object(link, 0)
         except Exception as error:
             msg = f"AXHypertext: Exception in get_child_at_offset: {error}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return None
 
         tokens = [f"AXHypertext: Child at offset {offset} in", obj, "is", child]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return child
 
     @staticmethod
-    def get_character_offset_in_parent(obj):
+    def get_character_offset_in_parent(obj: Atspi.Accessible) -> int:
         """Returns the offset of the embedded-object obj in the text of its parent."""
 
         if not AXObject.supports_text(AXObject.get_parent(obj)):
