@@ -330,6 +330,10 @@ KB_HEADER_KEY_BINDING = _("Key Binding")
 # to, for instance, web browsing.
 KB_GROUP_DEFAULT = C_("keybindings", "Default")
 
+# Translators: This string is a label for the group of Orca commands which
+# are related to debugging.
+KB_GROUP_DEBUGGING_TOOLS = C_("keybindings", "Debugging Tools")
+
 # Translators: This string is a label for the group of Orca commands which
 # are related to its "learn mode". Please use the same translation as done
 # in cmdnames.py
@@ -350,6 +354,10 @@ KB_GROUP_BRAILLE = _("Braille Bindings")
 # are related to saving and jumping among objects via "bookmarks".
 KB_GROUP_BOOKMARKS = _("Bookmarks")
 
+# Translators: This string is a label for the group of Orca commands which
+# are related to the clipboard.
+KB_GROUP_CLIPBOARD = _("Clipboard")
+
 # Translators: This string is a label for the group of Orca commands which
 # are related to presenting the date and time.
 KB_GROUP_DATE_AND_TIME = _("Date and time")
@@ -395,10 +403,6 @@ KB_GROUP_TABLE_NAVIGATION = _("Table navigation")
 # text in the currently-focused object; etc.
 KB_GROUP_WHERE_AM_I = _("Object details")
 
-# Translators: This string is a label for the group of Orca commands which
-# do not currently have an associated key binding.
-KB_GROUP_UNBOUND = _("Unbound")
-
 # Translators: This string is a label for the group of Orca commands which
 # are related to Orca's "flat review" feature. This feature allows the blind
 # user to explore the text in a window in a 2D fashion. That is, Orca treats
