@@ -24,6 +24,9 @@
 
 """Provides utilities for managing input events."""
 
+# This has to be the first non-docstring line in the module to make linters happy.
+from __future__ import annotations
+
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
@@ -31,6 +34,8 @@ __copyright__ = "Copyright (c) 2024 Igalia, S.L." \
                 "Copyright (c) 2024 GNOME Foundation Inc."
 __license__   = "LGPL"
 
+from typing import TYPE_CHECKING
+
 import gi
 gi.require_version("Atspi", "2.0")
 gi.require_version("Gdk", "3.0")
@@ -45,31 +50,38 @@ from . import settings
 from .ax_object import AXObject
 from .ax_utilities import AXUtilities
 
+if TYPE_CHECKING:
+    from . import keybindings
 
 class InputEventManager:
     """Provides utilities for managing input events."""
 
-    def __init__(self):
-        self._last_input_event = None
-        self._last_non_modifier_key_event = None
-        self._device = None
+    def __init__(self) -> None:
+        self._last_input_event: input_event.InputEvent | None = None
+        self._last_non_modifier_key_event: input_event.KeyboardEvent | None = None
+        self._device: Atspi.Device | None = None
+        self._mapped_keycodes: list[int] = []
+        self._mapped_keysyms: list[int] = []
 
-    def start_key_watcher(self):
+    def start_key_watcher(self) -> None:
         """Starts the watcher for keyboard input events."""
 
         msg = "INPUT EVENT MANAGER: Starting key watcher."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        self._device = Atspi.Device.new()
+        debug.print_message(debug.LEVEL_INFO, msg, True)
+        if Atspi.get_version() >= (2, 55, 90):
+            self._device = Atspi.Device.new_full("org.gnome.Orca")
+        else:
+            self._device = Atspi.Device.new()
         self._device.add_key_watcher(self.process_keyboard_event)
 
-    def stop_key_watcher(self):
+    def stop_key_watcher(self) -> None:
         """Starts the watcher for keyboard input events."""
 
         msg = "INPUT EVENT MANAGER: Stopping key watcher."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         self._device = None
 
-    def add_grabs_for_keybinding(self, binding):
+    def add_grabs_for_keybinding(self, binding: keybindings.KeyBinding) -> list[int]:
         """Adds grabs for binding if it is enabled, returns grab IDs."""
 
         if not (binding.is_enabled() and binding.is_bound()):
@@ -77,12 +89,12 @@ class InputEventManager:
 
         if binding.has_grabs():
             tokens = ["INPUT EVENT MANAGER:", binding, "already has grabs."]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return []
 
         if self._device is None:
             tokens = ["INPUT EVENT MANAGER: No device to add grab for", binding]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return []
 
         grab_ids = []
@@ -91,81 +103,111 @@ class InputEventManager:
 
         return grab_ids
 
-    def remove_grabs_for_keybinding(self, binding):
+    def remove_grabs_for_keybinding(self, binding: keybindings.KeyBinding) -> None:
         """Removes grabs for binding."""
 
         if self._device is None:
             tokens = ["INPUT EVENT MANAGER: No device to remove grab from", binding]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return
 
         grab_ids = binding.get_grab_ids()
         if not grab_ids:
             tokens = ["INPUT EVENT MANAGER:", binding, "doesn't have grabs to remove."]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return
 
         for grab_id in grab_ids:
             self._device.remove_key_grab(grab_id)
 
-    def map_keycode_to_modifier(self, keycode):
+    def map_keycode_to_modifier(self, keycode: int) -> int:
         """Maps keycode as a modifier, returns the newly-mapped modifier."""
 
         if self._device is None:
             msg = f"INPUT EVENT MANAGER: No device to map keycode {keycode} to modifier"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return 0
 
+        self._mapped_keycodes.append(keycode)
         return self._device.map_modifier(keycode)
 
-    def add_grab_for_modifier(self, modifier, keycode):
+    def map_keysym_to_modifier(self, keysym: int) -> int:
+        """Maps keysym as a modifier, returns the newly-mapped modifier."""
+
+        if self._device is None:
+            msg = f"INPUT EVENT MANAGER: No device to map keysym {keysym} to modifier"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return 0
+
+        self._mapped_keysyms.append(keysym)
+        return self._device.map_keysym_modifier(keysym)
+
+    def unmap_all_modifiers(self) -> None:
+        """Unmaps all previously mapped modifiers."""
+
+        if self._device is None:
+            msg = "INPUT EVENT MANAGER: No device to unmap all modifiers from"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return
+
+        for keycode in self._mapped_keycodes:
+            self._device.unmap_modifier(keycode)
+
+        for keysym in self._mapped_keysyms:
+            self._device.unmap_keysym_modifier(keysym)
+
+        self._mapped_keycodes.clear()
+        self._mapped_keysyms.clear()
+
+    def add_grab_for_modifier(self, modifier: str, keysym: int, keycode: int) -> int:
         """Adds grab for modifier, returns grab id."""
 
         if self._device is None:
-            tokens = ["INPUT EVENT MANAGER: No device to add grab for", modifier]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            msg = f"INPUT EVENT MANAGER: No device to add grab for {modifier}"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return -1
 
         kd = Atspi.KeyDefinition()
+        kd.keysym = keysym
         kd.keycode = keycode
         kd.modifiers = 0
         grab_id = self._device.add_key_grab(kd)
 
-        tokens = ["INPUT EVENT MANAGER: Grab id for", modifier, ":", grab_id]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        msg = f"INPUT EVENT MANAGER: Grab id for {modifier}: {grab_id}"
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         return grab_id
 
-    def remove_grab_for_modifier(self, modifier, grab_id):
+    def remove_grab_for_modifier(self, modifier: str, grab_id: int) -> None:
         """Removes grab for modifier."""
 
         if self._device is None:
-            tokens = ["INPUT EVENT MANAGER: No device to remove grab from", modifier]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            msg = f"INPUT EVENT MANAGER: No device to remove grab from {modifier}"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         self._device.remove_key_grab(grab_id)
-        tokens = ["INPUT EVENT MANAGER: Grab ID removed for", modifier, ":", grab_id]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        msg = f"INPUT EVENT MANAGER: Grab id removed for {modifier}: {grab_id}"
+        debug.print_message(debug.LEVEL_INFO, msg, True)
 
-    def grab_keyboard(self, reason=""):
+    def grab_keyboard(self, reason: str = "") -> None:
         """Grabs the keyboard, e.g. when entering learn mode."""
 
         msg = "INPUT EVENT MANAGER: Grabbing keyboard"
         if reason:
             msg += f" Reason: {reason}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         Atspi.Device.grab_keyboard(self._device)
 
-    def ungrab_keyboard(self, reason=""):
+    def ungrab_keyboard(self, reason: str = "") -> None:
         """Removes keyboard grab, e.g. when exiting learn mode."""
 
         msg = "INPUT EVENT MANAGER: Ungrabbing keyboard"
         if reason:
             msg += f" Reason: {reason}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         Atspi.Device.ungrab_keyboard(self._device)
 
-    def process_braille_event(self, event):
+    def process_braille_event(self, event: Atspi.Event) -> bool:
         """Processes this Braille event."""
 
         braille_event = input_event.BrailleEvent(event)
@@ -174,7 +216,7 @@ class InputEventManager:
         self._last_non_modifier_key_event = None
         return result
 
-    def process_mouse_button_event(self, event):
+    def process_mouse_button_event(self, event: Atspi.Event) -> None:
         """Processes this Mouse event."""
 
         mouse_event = input_event.MouseButtonEvent(event)
@@ -182,27 +224,43 @@ class InputEventManager:
         self._last_input_event = mouse_event
 
     # pylint: disable=too-many-arguments
-    def process_keyboard_event(self, _device, pressed, keycode, keysym, modifiers, text):
+    # pylint: disable=too-many-positional-arguments
+    def process_keyboard_event(
+        self,
+        _device: Atspi.Device,
+        pressed: bool,
+        keycode: int,
+        keysym: int,
+        modifiers: int,
+        text: str
+    ) -> bool:
         """Processes this Atspi keyboard event."""
 
         event = input_event.KeyboardEvent(pressed, keycode, keysym, modifiers, text)
         if event in [self._last_input_event, self._last_non_modifier_key_event]:
             msg = "INPUT EVENT MANAGER: Received duplicate event."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return False
 
         manager = focus_manager.get_manager()
         if pressed:
             window = manager.get_active_window()
-            if not manager.can_be_active_window(window):
-                window = manager.find_active_window()
-                tokens = ["INPUT EVENT MANAGER: Updating window and active window to", window]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                manager.set_active_window(window)
+            if not AXUtilities.can_be_active_window(window):
+                new_window = AXUtilities.find_active_window()
+                if new_window is not None:
+                    window = new_window
+                    tokens = ["INPUT EVENT MANAGER: Updating window and active window to", window]
+                    debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+                    manager.set_active_window(window)
+                else:
+                    # One example: Brave's popup menus live in frames which lack the active state.
+                    tokens = ["WARNING:", window, "cannot be active window. No alternative found."]
+                    debug.print_tokens(debug.LEVEL_WARNING, tokens, True)
             event.set_window(window)
             event.set_object(manager.get_locus_of_focus())
             event.set_script(script_manager.get_manager().get_active_script())
         elif self.last_event_was_keyboard():
+            assert isinstance(self._last_input_event, input_event.KeyboardEvent)
             event.set_window(self._last_input_event.get_window())
             event.set_object(self._last_input_event.get_object())
             event.set_script(self._last_input_event.get_script())
@@ -212,19 +270,22 @@ class InputEventManager:
             event.set_script(script_manager.get_manager().get_active_script())
 
         event.set_click_count(self._determine_keyboard_event_click_count(event))
-        result = event.process()
+        event.process()
 
         if event.is_modifier_key():
             if self.is_release_for(event, self._last_input_event):
                 msg = "INPUT EVENT MANAGER: Clearing last non modifier key event"
-                debug.printMessage(debug.LEVEL_INFO, msg, True)
+                debug.print_message(debug.LEVEL_INFO, msg, True)
                 self._last_non_modifier_key_event = None
         else:
             self._last_non_modifier_key_event = event
         self._last_input_event = event
-        return result
+        return True
 
-    def _determine_keyboard_event_click_count(self, event):
+    # pylint: enable=too-many-arguments
+    # pylint: enable=too-many-positional-arguments
+
+    def _determine_keyboard_event_click_count(self, event: input_event.KeyboardEvent) -> int:
         """Determines the click count of event."""
 
         if not self.last_event_was_keyboard():
@@ -235,6 +296,7 @@ class InputEventManager:
         else:
             last_event = self._last_non_modifier_key_event or self._last_input_event
 
+        assert isinstance(last_event, input_event.KeyboardEvent)
         if (event.time - last_event.time > settings.doubleClickTimeout) or \
            (event.keyval_name != last_event.keyval_name) or \
            (event.get_object() != last_event.get_object()):
@@ -249,11 +311,13 @@ class InputEventManager:
             return 1
         return last_count + 1
 
-    def _determine_mouse_event_click_count(self, event):
+    def _determine_mouse_event_click_count(self, event: input_event.MouseButtonEvent) -> int:
         """Determines the click count of event."""
 
         if not self.last_event_was_mouse_button():
             return 1
+
+        assert isinstance(self._last_input_event, input_event.MouseButtonEvent)
         if not event.pressed:
             return self._last_input_event.get_click_count()
         if self._last_input_event.button != event.button:
@@ -263,12 +327,12 @@ class InputEventManager:
 
         return self._last_input_event.get_click_count() + 1
 
-    def last_event_was_keyboard(self):
+    def last_event_was_keyboard(self) -> bool:
         """Returns True if the last event is a keyboard event."""
 
         return isinstance(self._last_input_event, input_event.KeyboardEvent)
 
-    def last_event_was_mouse_button(self):
+    def last_event_was_mouse_button(self) -> bool:
         """Returns True if the last event is a mouse button event."""
 
         return isinstance(self._last_input_event, input_event.MouseButtonEvent)
@@ -297,7 +361,7 @@ class InputEventManager:
             f"INPUT EVENT MANAGER: {event1.as_single_line_string()} "
             f"is release for {event2.as_single_line_string()}: {result}"
         )
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         return result
 
     def last_event_equals_or_is_release_for_event(self, event):
@@ -344,10 +408,12 @@ class InputEventManager:
     def last_event_was_command(self):
         """Returns True if the last event is believed to be a command."""
 
-        rv = bool(self._last_key_and_modifiers()[1] & 1 << Atspi.ModifierType.CONTROL)
-        msg = f"INPUT EVENT MANAGER: Last event was command: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if bool(self._last_key_and_modifiers()[1] & 1 << Atspi.ModifierType.CONTROL):
+            msg = "INPUT EVENT MANAGER: Last event was command."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+
+        return False
 
     def last_event_was_shortcut_for(self, obj):
         """Returns True if the last event is believed to be a shortcut key for obj."""
@@ -363,8 +429,9 @@ class InputEventManager:
                 rv = True
                 break
 
-        tokens = ["INPUT EVENT MANAGER: Last event was shortcut for", obj, rv]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        if rv:
+            tokens = ["INPUT EVENT MANAGER: Last event was shortcut for", obj]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return rv
 
     def last_event_was_printable_key(self):
@@ -373,10 +440,12 @@ class InputEventManager:
         if not self.last_event_was_keyboard():
             return False
 
-        rv = self._last_input_event.is_printable_key()
-        msg = f"INPUT EVENT MANAGER: Last event was printable key: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.is_printable_key():
+            msg = "INPUT EVENT MANAGER: Last event was printable key"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+
+        return False
 
     def last_event_was_caret_navigation(self):
         """Returns True if the last event is believed to be caret navigation."""
@@ -397,8 +466,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.SHIFT)
 
-        msg = f"INPUT EVENT MANAGER: Last event was caret selection: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was caret selection"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_backward_caret_navigation(self):
@@ -410,8 +480,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.SHIFT
 
-        msg = f"INPUT EVENT MANAGER: Last event was backward caret navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was backward caret navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_forward_caret_navigation(self):
@@ -423,8 +494,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.SHIFT
 
-        msg = f"INPUT EVENT MANAGER: Last event was forward caret navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was forward caret navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_forward_caret_selection(self):
@@ -436,8 +508,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.SHIFT)
 
-        msg = f"INPUT EVENT MANAGER: Last event was forward caret selection: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was forward caret selection"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_character_navigation(self):
@@ -451,8 +524,9 @@ class InputEventManager:
         else:
             rv = True
 
-        msg = f"INPUT EVENT MANAGER: Last event was character navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was character navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_word_navigation(self):
@@ -464,8 +538,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.CONTROL)
 
-        msg = f"INPUT EVENT MANAGER: Last event was word navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was word navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_previous_word_navigation(self):
@@ -477,8 +552,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.CONTROL)
 
-        msg = f"INPUT EVENT MANAGER: Last event was previous-word navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was previous-word navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_next_word_navigation(self):
@@ -490,8 +566,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.CONTROL)
 
-        msg = f"INPUT EVENT MANAGER: Last event was next-word navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was next-word navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_line_navigation(self):
@@ -504,10 +581,14 @@ class InputEventManager:
             rv = False
         else:
             focus = focus_manager.get_manager().get_locus_of_focus()
-            rv = not AXObject.find_ancestor(focus, AXUtilities.is_combo_box)
+            if AXUtilities.is_single_line(focus):
+                rv = False
+            else:
+                rv = not AXUtilities.is_widget_controlled_by_line_navigation(focus)
 
-        msg = f"INPUT EVENT MANAGER: Last event was line navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was line navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_paragraph_navigation(self):
@@ -519,8 +600,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.SHIFT
 
-        msg = f"INPUT EVENT MANAGER: Last event was paragraph navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was paragraph navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_line_boundary_navigation(self):
@@ -532,8 +614,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.CONTROL
 
-        msg = f"INPUT EVENT MANAGER: Last event was line boundary navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was line boundary navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_file_boundary_navigation(self):
@@ -545,8 +628,9 @@ class InputEventManager:
         else:
             rv = bool(mods & 1 << Atspi.ModifierType.CONTROL)
 
-        msg = f"INPUT EVENT MANAGER: Last event was file boundary navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was file boundary navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_page_navigation(self):
@@ -559,10 +643,14 @@ class InputEventManager:
             rv = False
         else:
             focus = focus_manager.get_manager().get_locus_of_focus()
-            rv = not AXObject.find_ancestor(focus, AXUtilities.is_combo_box)
+            if AXUtilities.is_single_line(focus):
+                rv = False
+            else:
+                rv = not AXUtilities.is_widget_controlled_by_line_navigation(focus)
 
-        msg = f"INPUT EVENT MANAGER: Last event was page navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was page navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_page_switch(self):
@@ -576,8 +664,9 @@ class InputEventManager:
         else:
             rv = False
 
-        msg = f"INPUT EVENT MANAGER: Last event was page switch: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was page switch"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_tab_navigation(self):
@@ -591,8 +680,9 @@ class InputEventManager:
         else:
             rv = True
 
-        msg = f"INPUT EVENT MANAGER: Last event was Tab navigation: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was Tab navigation"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_table_sort(self):
@@ -608,8 +698,9 @@ class InputEventManager:
         else:
             rv = False
 
-        msg = f"INPUT EVENT MANAGER: Last event was table sort: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was table sort"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_unmodified_arrow(self):
@@ -719,8 +810,9 @@ class InputEventManager:
         else:
             rv = False
 
-        msg = f"INPUT EVENT MANAGER: Last event was delete: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was delete"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_cut(self):
@@ -728,15 +820,14 @@ class InputEventManager:
 
         keycode, mods = self._last_keycode_and_modifiers()
         keynames = self._all_names_for_key_code(keycode)
-        if "x" in keynames:
-            rv = mods & 1 << Atspi.ModifierType.CONTROL \
-                and not mods & 1 << Atspi.ModifierType.SHIFT
-        else:
-            rv = False
+        if "x" not in keynames:
+            return False
 
-        msg = f"INPUT EVENT MANAGER: Last event was cut: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT:
+            msg = "INPUT EVENT MANAGER: Last event was cut"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_copy(self):
         """Returns True if the last event is believed to be the copy command."""
@@ -750,8 +841,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.SHIFT
 
-        msg = f"INPUT EVENT MANAGER: Last event was copy: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was copy"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_paste(self):
@@ -766,8 +858,9 @@ class InputEventManager:
         else:
             rv = not mods & 1 << Atspi.ModifierType.SHIFT
 
-        msg = f"INPUT EVENT MANAGER: Last event was paste: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was paste"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_undo(self):
@@ -775,15 +868,13 @@ class InputEventManager:
 
         keycode, mods = self._last_keycode_and_modifiers()
         keynames = self._all_names_for_key_code(keycode)
-        if "z" in keynames:
-            rv = mods & 1 << Atspi.ModifierType.CONTROL \
-                and not mods & 1 << Atspi.ModifierType.SHIFT
-        else:
-            rv = False
-
-        msg = f"INPUT EVENT MANAGER: Last event was undo: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if "z" not in keynames:
+            return False
+        if mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT:
+            msg = "INPUT EVENT MANAGER: Last event was undo"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_redo(self):
         """Returns True if the last event is believed to be the redo command."""
@@ -799,8 +890,9 @@ class InputEventManager:
         else:
             rv = False
 
-        msg = f"INPUT EVENT MANAGER: Last event was redo: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if rv:
+            msg = "INPUT EVENT MANAGER: Last event was redo"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
         return rv
 
     def last_event_was_select_all(self):
@@ -808,15 +900,14 @@ class InputEventManager:
 
         keycode, mods = self._last_keycode_and_modifiers()
         keynames = self._all_names_for_key_code(keycode)
-        if "a" in keynames:
-            rv = mods & 1 << Atspi.ModifierType.CONTROL \
-                and not mods & 1 << Atspi.ModifierType.SHIFT
-        else:
-            rv = False
+        if "a" not in keynames:
+            return False
 
-        msg = f"INPUT EVENT MANAGER: Last event was select all: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if (mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT):
+            msg = "INPUT EVENT MANAGER: Last event was select all"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_primary_click(self):
         """Returns True if the last event is a primary mouse click."""
@@ -824,10 +915,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "1" and self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was primary click: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "1" and self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was primary click"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_primary_release(self):
         """Returns True if the last event is a primary mouse release."""
@@ -835,10 +927,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "1" and not self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was primary release: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "1" and not self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was primary release"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_primary_click_or_release(self):
         """Returns True if the last event is a primary mouse click or release."""
@@ -846,10 +939,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "1"
-        msg = f"INPUT EVENT MANAGER: Last event was primary click or release: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "1":
+            msg = "INPUT EVENT MANAGER: Last event was primary click or release"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_middle_click(self):
         """Returns True if the last event is a middle mouse click."""
@@ -857,10 +951,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "2" and self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was middle click: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "2" and self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was middle click"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_middle_release(self):
         """Returns True if the last event is a middle mouse release."""
@@ -868,10 +963,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "2" and not self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was middle release: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "2" and not self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was middle release"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_secondary_click(self):
         """Returns True if the last event is a secondary mouse click."""
@@ -879,10 +975,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "3" and self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was secondary click: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "3" and self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was secondary click"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
     def last_event_was_secondary_release(self):
         """Returns True if the last event is a secondary mouse release."""
@@ -890,10 +987,11 @@ class InputEventManager:
         if not self.last_event_was_mouse_button():
             return False
 
-        rv = self._last_input_event.button == "3" and not self._last_input_event.pressed
-        msg = f"INPUT EVENT MANAGER: Last event was secondary release: {rv}"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return rv
+        if self._last_input_event.button == "3" and not self._last_input_event.pressed:
+            msg = "INPUT EVENT MANAGER: Last event was secondary release"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return True
+        return False
 
 
 _manager = InputEventManager()
