@@ -18,14 +18,25 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# pylint: disable=wrong-import-position
+
 """Provides ability to navigate objects hierarchically."""
 
+# This has to be the first non-docstring line in the module to make linters happy.
+from __future__ import annotations
+
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2023 The Orca Team"
 __license__   = "LGPL"
 
+from typing import Optional, TYPE_CHECKING
+
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
+
 from . import cmdnames
 from . import debug
 from . import focus_manager
@@ -36,41 +47,45 @@ from .ax_event_synthesizer import AXEventSynthesizer
 from .ax_object import AXObject
 from .ax_utilities import AXUtilities
 
+if TYPE_CHECKING:
+    from .scripts import default
 
 class ObjectNavigator:
     """Provides ability to navigate objects hierarchically."""
 
-    def __init__(self):
-        self._navigator_focus = None
-        self._last_navigator_focus = None
-        self._last_locus_of_focus = None
-        self._simplify = True
-        self._handlers = self.get_handlers(True)
-        self._bindings = keybindings.KeyBindings()
+    def __init__(self) -> None:
+        self._navigator_focus: Optional[Atspi.Accessible] = None
+        self._last_navigator_focus: Optional[Atspi.Accessible] = None
+        self._last_locus_of_focus: Optional[Atspi.Accessible] = None
+        self._simplify: bool = True
+        self._handlers: dict = self.get_handlers(True)
+        self._bindings: keybindings.KeyBindings = keybindings.KeyBindings()
 
-    def get_bindings(self, refresh=False, is_desktop=True):
+    def get_bindings(
+        self, refresh: bool = False, is_desktop: bool = True
+    ) -> keybindings.KeyBindings:
         """Returns the object-navigator keybindings."""
 
         if refresh:
-            msg = "OBJECT NAVIGATOR: Refreshing bindings."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            msg = f"OBJECT NAVIGATOR: Refreshing bindings. Is desktop: {is_desktop}"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             self._setup_bindings()
-        elif self._bindings.isEmpty():
+        elif self._bindings.is_empty():
             self._setup_bindings()
 
         return self._bindings
 
-    def get_handlers(self, refresh=False):
+    def get_handlers(self, refresh: bool = False) -> dict[str, input_event.InputEventHandler]:
         """Returns the object-navigator handlers."""
 
         if refresh:
             msg = "OBJECT NAVIGATOR: Refreshing handlers."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             self._setup_handlers()
 
         return self._handlers
 
-    def _setup_bindings(self):
+    def _setup_bindings(self) -> None:
         """Sets up the object-navigator key bindings."""
 
         self._bindings = keybindings.KeyBindings()
@@ -78,49 +93,49 @@ class ObjectNavigator:
         self._bindings.add(
             keybindings.KeyBinding(
                 "Up",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_up")))
+                self._handlers["object_navigator_up"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "Down",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_down")))
+                self._handlers["object_navigator_down"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "Right",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_next")))
+                self._handlers["object_navigator_next"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "Left",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_previous")))
+                self._handlers["object_navigator_previous"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "Return",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_perform_action")))
+                self._handlers["object_navigator_perform_action"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "s",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_CTRL_MODIFIER_MASK,
-                self._handlers.get("object_navigator_toggle_simplify")))
+                self._handlers["object_navigator_toggle_simplify"]))
 
         msg = "OBJECT NAVIGATOR: Bindings set up."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
 
-    def _setup_handlers(self):
+    def _setup_handlers(self) -> None:
         """Sets up the object-navigator input event handlers."""
 
         self._handlers = {}
@@ -156,38 +171,40 @@ class ObjectNavigator:
                 cmdnames.NAVIGATOR_TOGGLE_SIMPLIFIED)
 
         msg = "OBJECT NAVIGATOR: Handlers set up."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
 
-    def _include_in_simple_navigation(self, obj):
+    def _include_in_simple_navigation(self, obj: Atspi.Accessible) -> bool:
         """Returns True if obj should be included in simple navigation."""
 
         return AXUtilities.is_paragraph(obj)
 
-    def _exclude_from_simple_navigation(self, script, obj):
+    def _exclude_from_simple_navigation(
+        self, _script: default.Script, obj: Atspi.Accessible
+    ) -> bool:
         """Returns True if obj should be excluded from simple navigation."""
 
         if self._include_in_simple_navigation(obj):
             tokens = ["OBJECT NAVIGATOR: Not excluding", obj, ": explicit inclusion"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return False
 
-        # isLayoutOnly should catch things that really should be skipped.
+        # is_layout_only should catch things that really should be skipped.
         #
         # You do not want to exclude all sections because they may be focusable, e.g.
         # <div tabindex=0>foo</div> should not be excluded, despite the poor authoring.
         #
         # You do not want to exclude table cells and headers because it will make the
         # selectable items in tables non-navigable (e.g. the mail folders in Evolution)
-        if script.utilities.isLayoutOnly(obj):
+        if AXUtilities.is_layout_only(obj):
             tokens = ["OBJECT NAVIGATOR: Excluding", obj, ": is layout only"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return True
 
         tokens = ["OBJECT NAVIGATOR: Not excluding", obj]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return False
 
-    def _children(self, script, obj):
+    def _children(self, script: default.Script, obj: Atspi.Accessible) -> list:
         """Returns a list of children for obj, taking simple navigation into account."""
 
         if not AXObject.get_child_count(obj):
@@ -207,7 +224,7 @@ class ObjectNavigator:
 
         return functional_children
 
-    def _parent(self, script, obj):
+    def _parent(self, script: default.Script, obj: Atspi.Accessible) -> Optional[Atspi.Accessible]:
         """Returns the parent for obj, taking simple navigation into account."""
 
         parent = AXObject.get_parent(obj)
@@ -220,17 +237,17 @@ class ObjectNavigator:
 
         return parent
 
-    def _set_navigator_focus(self, obj):
+    def _set_navigator_focus(self, obj: Atspi.Accessible) -> None:
         """Changes the navigator focus, storing the previous focus."""
 
         self._last_navigator_focus = self._navigator_focus
         self._navigator_focus = obj
 
-    def update(self):
+    def update(self) -> None:
         """Updates the navigator focus to Orca's object of interest."""
 
-        mode, region = focus_manager.getManager().get_active_mode_and_object_of_interest()
-        obj = region or focus_manager.getManager().get_locus_of_focus()
+        mode, region = focus_manager.get_manager().get_active_mode_and_object_of_interest()
+        obj = region or focus_manager.get_manager().get_locus_of_focus()
         if self._last_locus_of_focus == obj \
            or (region is None and mode == focus_manager.FLAT_REVIEW):
             return
@@ -238,16 +255,16 @@ class ObjectNavigator:
         self._navigator_focus = obj
         self._last_locus_of_focus = obj
 
-    def present(self, script):
+    def present(self, script: default.Script) -> None:
         """Presents the current navigator focus to the user."""
 
         tokens = ["OBJECT NAVIGATOR: Presenting", self._navigator_focus]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
-        focus_manager.getManager().emit_region_changed(
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+        focus_manager.get_manager().emit_region_changed(
             self._navigator_focus, mode=focus_manager.OBJECT_NAVIGATOR)
         script.presentObject(self._navigator_focus, priorObj=self._last_navigator_focus)
 
-    def up(self, script, event=None):
+    def up(self, script: default.Script, _event: Optional[input_event.InputEvent] = None) -> bool:
         """Moves the navigator focus to the parent of the current focus."""
 
         self.update()
@@ -257,27 +274,29 @@ class ObjectNavigator:
             self.present(script)
         else:
             script.presentMessage(messages.NAVIGATOR_NO_PARENT)
+        return True
 
-    def down(self, script, event=None):
+    def down(self, script: default.Script, _event: Optional[input_event.InputEvent] = None) -> bool:
         """Moves the navigator focus to the first child of the current focus."""
 
         self.update()
         children = self._children(script, self._navigator_focus)
         if not children:
             script.presentMessage(messages.NAVIGATOR_NO_CHILDREN)
-            return
+            return True
 
         self._set_navigator_focus(children[0])
         self.present(script)
+        return True
 
-    def next(self, script, event=None):
+    def next(self, script: default.Script, _event: Optional[input_event.InputEvent] = None) -> bool:
         """Moves the navigator focus to the next sibling of the current focus."""
 
         self.update()
         parent = self._parent(script, self._navigator_focus)
         if parent is None:
             script.presentMessage(messages.NAVIGATOR_NO_NEXT)
-            return
+            return True
 
         siblings = self._children(script, parent)
         if self._navigator_focus in siblings:
@@ -290,15 +309,18 @@ class ObjectNavigator:
         else:
             self._set_navigator_focus(parent)
             self.present(script)
+        return True
 
-    def previous(self, script, event=None):
+    def previous(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Moves the navigator focus to the previous sibling of the current focus."""
 
         self.update()
         parent = self._parent(script, self._navigator_focus)
         if parent is None:
             script.presentMessage(messages.NAVIGATOR_NO_PREVIOUS)
-            return
+            return True
 
         siblings = self._children(script, parent)
         if self._navigator_focus in siblings:
@@ -311,8 +333,11 @@ class ObjectNavigator:
         else:
             self._set_navigator_focus(parent)
             self.present(script)
+        return True
 
-    def toggle_simplify(self, script, event=None):
+    def toggle_simplify(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Toggles simplified navigation."""
 
         self._simplify = not self._simplify
@@ -322,7 +347,9 @@ class ObjectNavigator:
             script.presentMessage(messages.NAVIGATOR_SIMPLIFIED_DISABLED)
         return True
 
-    def perform_action(self, script, event=None):
+    def perform_action(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Attempts to click on the current focus."""
         if AXEventSynthesizer.try_all_clickable_actions(self._navigator_focus):
             return True
@@ -331,8 +358,8 @@ class ObjectNavigator:
         return True
 
 
-_navigator = ObjectNavigator()
-def getNavigator():
+_navigator: ObjectNavigator = ObjectNavigator()
+def get_navigator() -> ObjectNavigator:
     """Returns the Object Navigator"""
 
     return _navigator
