@@ -17,8 +17,10 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
-""" Custom script for Thunderbird 3.
-"""
+# pylint: disable=invalid-name
+# pylint: disable=duplicate-code
+
+"""Custom script for Thunderbird."""
 
 __id__        = "$Id$"
 __version__   = "$Revision$"
@@ -26,4 +28,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+# https://gitlab.gnome.org/GNOME/orca/-/issues/358
+# ruff: noqa: F401
 from .script import Script
