@@ -57,7 +57,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.locus_of_focus_changed(self, event, old_focus, new_focus)
 
     def on_active_changed(self, event):
@@ -67,11 +67,11 @@ class Script(web.Script):
             return
 
         if event.detail1 and AXUtilities.is_frame(event.source) \
-           and not focus_manager.get_manager().can_be_active_window(event.source):
+           and not AXUtilities.can_be_active_window(event.source):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_active_changed(self, event)
 
     def on_busy_changed(self, event):
@@ -79,39 +79,40 @@ class Script(web.Script):
 
         if AXComponent.has_no_size(event.source):
             msg = "CHROMIUM: Ignoring event from page with no size."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_busy_changed(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_busy_changed(self, event)
 
     def on_caret_moved(self, event):
         """Callback for object:text-caret-moved accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.source):
-            msg = "CHROMIUM: Ignoring event from static-text leaf"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if not AXUtilities.is_web_element(event.source) \
+           and AXUtilities.is_web_element(AXObject.get_parent(event.source)):
+            msg = "CHROMIUM: Ignoring because source is not an element"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if self.utilities.isRedundantAutocompleteEvent(event):
             msg = "CHROMIUM: Ignoring redundant autocomplete event"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_caret_moved(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_caret_moved(self, event)
 
     def on_checked_changed(self, event):
@@ -121,7 +122,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_checked_changed(self, event)
 
     def on_column_reordered(self, event):
@@ -131,37 +132,39 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_column_reordered(self, event)
 
     def on_children_added(self, event):
         """Callback for object:children-changed:add accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.any_data):
-            msg = "CHROMIUM: Ignoring because child is static text leaf"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if AXUtilities.is_web_element(event.source) \
+           and not AXUtilities.is_web_element(event.any_data):
+            msg = "CHROMIUM: Ignoring because child is not an element"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_children_added(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_children_added(self, event)
 
     def on_children_removed(self, event):
         """Callback for object:children-changed:removed accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.any_data):
-            msg = "CHROMIUM: Ignoring because child is static text leaf"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if AXUtilities.is_web_element(event.source) \
+           and not AXUtilities.is_web_element(event.any_data):
+            msg = "CHROMIUM: Ignoring because child is not an element"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_children_removed(event):
             return
 
         msg = "Chromium: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_children_removed(self, event)
 
     def on_document_load_complete(self, event):
@@ -169,14 +172,14 @@ class Script(web.Script):
 
         if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_document_load_complete(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_document_load_complete(self, event)
 
     def on_document_load_stopped(self, event):
@@ -184,14 +187,14 @@ class Script(web.Script):
 
         if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_document_load_stopped(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_document_load_stopped(self, event)
 
     def on_document_reload(self, event):
@@ -199,14 +202,14 @@ class Script(web.Script):
 
         if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_document_reload(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_document_reload(self, event)
 
     def on_expanded_changed(self, event):
@@ -216,7 +219,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_expanded_changed(self, event)
 
     def on_focused_changed(self, event):
@@ -225,14 +228,14 @@ class Script(web.Script):
         if self.utilities.isDocument(event.source) \
            and not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from document with no URI."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_focused_changed(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_focused_changed(self, event)
 
     def on_mouse_button(self, event):
@@ -242,7 +245,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_mouse_button(self, event)
 
     def on_name_changed(self, event):
@@ -252,7 +255,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_name_changed(self, event)
 
     def on_row_reordered(self, event):
@@ -262,7 +265,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_row_reordered(self, event)
 
     def on_selected_changed(self, event):
@@ -272,7 +275,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_selected_changed(self, event)
 
     def on_selection_changed(self, event):
@@ -282,7 +285,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_selection_changed(self, event)
 
     def on_showing_changed(self, event):
@@ -292,7 +295,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_showing_changed(self, event)
 
     def on_text_attributes_changed(self, event):
@@ -302,7 +305,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_text_attributes_changed(self, event)
 
     def on_text_deleted(self, event):
@@ -312,7 +315,7 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_text_deleted(self, event)
 
     def on_text_inserted(self, event):
@@ -322,33 +325,29 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_text_inserted(self, event)
 
     def on_text_selection_changed(self, event):
         """Callback for object:text-selection-changed accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.source):
-            msg = "CHROMIUM: Ignoring event from static-text leaf"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
-            return
-
-        if self.utilities.isListItemMarker(event.source):
-            msg = "CHROMIUM: Ignoring event from list item marker"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        if not AXUtilities.is_web_element(event.source) \
+           and AXUtilities.is_web_element(AXObject.get_parent(event.source)):
+            msg = "CHROMIUM: Ignoring because source is not an element"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return
 
         if super().on_text_selection_changed(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_text_selection_changed(self, event)
 
     def on_window_activated(self, event):
         """Callback for window:activate accessibility events."""
 
-        if not focus_manager.get_manager().can_be_active_window(event.source):
+        if not AXUtilities.can_be_active_window(event.source):
             return
 
         # If this is a frame for a popup menu, we don't want to treat
@@ -370,19 +369,19 @@ class Script(web.Script):
                     focus_manager.get_manager().set_locus_of_focus(event, menu, False)
 
                 tokens = ["CHROMIUM: Setting locusOfFocus to active item", activeItem]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+                debug.print_tokens(debug.LEVEL_INFO, tokens, True)
                 focus_manager.get_manager().set_locus_of_focus(event, activeItem)
                 return
 
             tokens = ["CHROMIUM: Setting locusOfFocus to popup menu", menu]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             focus_manager.get_manager().set_locus_of_focus(event, menu)
 
         if super().on_window_activated(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_window_activated(self, event)
 
         # Right now we don't get accessibility events for alerts which are
@@ -399,5 +398,5 @@ class Script(web.Script):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
         default.Script.on_window_deactivated(self, event)
