@@ -18,10 +18,13 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
-# pylint: disable=unused-argument
+# pylint: disable=too-many-public-methods
 
 """Configures speech and verbosity settings and adjusts strings accordingly."""
 
+# This must be the first non-docstring line in the module to make linters happy.
+from __future__ import annotations
+
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
@@ -30,6 +33,7 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc." \
 __license__   = "LGPL"
 
 import re
+from typing import Optional, TYPE_CHECKING
 
 from . import cmdnames
 from . import debug
@@ -42,37 +46,48 @@ from . import settings_manager
 from . import speech
 from .ax_hypertext import AXHypertext
 from .ax_table import AXTable
+from .ax_utilities import AXUtilities
+
+if TYPE_CHECKING:
+    import gi
+    gi.require_version("Atspi", "2.0")
+    from gi.repository import Atspi
+
+    from .scripts import default
+    from .speechserver import SpeechServer
 
 class SpeechAndVerbosityManager:
     """Configures speech and verbosity settings and adjusts strings accordingly."""
 
-    def __init__(self):
-        self._handlers = self.get_handlers(True)
-        self._bindings = keybindings.KeyBindings()
+    def __init__(self) -> None:
+        self._handlers: dict[str, input_event.InputEventHandler] = self.get_handlers(True)
+        self._bindings: keybindings.KeyBindings = keybindings.KeyBindings()
 
-    def get_bindings(self, refresh=False, is_desktop=True):
+    def get_bindings(
+        self, refresh: bool = False, is_desktop: bool = True
+    ) -> keybindings.KeyBindings:
         """Returns the speech and verbosity manager keybindings."""
 
         if refresh:
             msg = f"SPEECH AND VERBOSITY MANAGER: Refreshing bindings.  Is desktop: {is_desktop}"
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             self._setup_bindings()
         elif self._bindings.is_empty():
             self._setup_bindings()
 
         return self._bindings
 
-    def get_handlers(self, refresh=False):
+    def get_handlers(self, refresh: bool = False) -> dict[str, input_event.InputEventHandler]:
         """Returns the speech and verbosity manager handlers."""
 
         if refresh:
             msg = "SPEECH AND VERBOSITY MANAGER: Refreshing handlers."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             self._setup_handlers()
 
         return self._handlers
 
-    def _setup_handlers(self):
+    def _setup_handlers(self) -> None:
         """Sets up the speech and verbosity input event handlers."""
 
         self._handlers = {}
@@ -153,9 +168,9 @@ class SpeechAndVerbosityManager:
                 cmdnames.INCREASE_SPEECH_VOLUME)
 
         msg = "SPEECH AND VERBOSITY MANAGER: Handlers set up."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
 
-    def _setup_bindings(self):
+    def _setup_bindings(self) -> None:
         """Sets up the speech and verbosity key bindings."""
 
         self._bindings = keybindings.KeyBindings()
@@ -165,227 +180,323 @@ class SpeechAndVerbosityManager:
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("cycleCapitalizationStyleHandler")))
+                self._handlers["cycleCapitalizationStyleHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("cycleSpeakingPunctuationLevelHandler")))
+                self._handlers["cycleSpeakingPunctuationLevelHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("cycleSynthesizerHandler")))
+                self._handlers["cycleSynthesizerHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("cycleKeyEchoHandler")))
+                self._handlers["cycleKeyEchoHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("changeNumberStyleHandler")))
+                self._handlers["changeNumberStyleHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("decreaseSpeechRateHandler")))
+                self._handlers["decreaseSpeechRateHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("increaseSpeechRateHandler")))
+                self._handlers["increaseSpeechRateHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("decreaseSpeechPitchHandler")))
+                self._handlers["decreaseSpeechPitchHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("increaseSpeechPitchHandler")))
+                self._handlers["increaseSpeechPitchHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("decreaseSpeechVolumeHandler")))
+                self._handlers["decreaseSpeechVolumeHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("increaseSpeechVolumeHandler")))
+                self._handlers["increaseSpeechVolumeHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("toggleSpeakingIndentationJustificationHandler")))
+                self._handlers["toggleSpeakingIndentationJustificationHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "s",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
-                self._handlers.get("toggleSilenceSpeechHandler")))
+                self._handlers["toggleSilenceSpeechHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "v",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
-                self._handlers.get("toggleSpeechVerbosityHandler")))
+                self._handlers["toggleSpeechVerbosityHandler"]))
 
         self._bindings.add(
             keybindings.KeyBinding(
                 "F11",
                 keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
-                self._handlers.get("toggleTableCellReadModeHandler")))
+                self._handlers["toggleTableCellReadModeHandler"]))
 
         msg = "SPEECH AND VERBOSITY MANAGER: Bindings set up."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.print_message(debug.LEVEL_INFO, msg, True)
 
-    def _get_server(self):
-        return speech.get_speech_server()
+    def _get_server(self) -> Optional[SpeechServer]:
+        result = speech.get_speech_server()
+        tokens = ["SPEECH AND VERBOSITY MANAGER: Speech server is", result]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+        return result
 
-    def decrease_rate(self, script, event=None):
+    def get_current_speech_server_info(self) -> tuple[str, str]:
+        """Returns the name and ID of the current speech server."""
+
+        # TODO - JD: The result is not in sync with the current output module. Should it be?
+        server = self._get_server()
+        if server is None:
+            return ("", "")
+
+        server_name, server_id = server.get_info()
+        msg = f"SPEECH AND VERBOSITY MANAGER: Speech server info: {server_name}, {server_id}."
+        debug.print_message(debug.LEVEL_INFO, msg, True)
+        return server_name, server_id
+
+    def check_speech_setting(self) -> None:
+        """Checks the speech setting and initializes speech if necessary."""
+
+        manager = settings_manager.get_manager()
+        if not manager.get_setting("enableSpeech"):
+            msg = "SPEECH AND VERBOSITY MANAGER: Speech is not enabled. Shutting down speech."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            self.shutdown_speech()
+            return
+
+        msg = "SPEECH AND VERBOSITY MANAGER: Speech is enabled."
+        debug.print_message(debug.LEVEL_INFO, msg, True)
+        self.start_speech()
+
+    def start_speech(self) -> None:
+        """Starts the speech server."""
+
+        speech.init()
+
+    def interrupt_speech(self) -> None:
+        """Interrupts the speech server."""
+
+        server = self._get_server()
+        if server is None:
+            return
+
+        server.stop()
+
+    def shutdown_speech(self) -> None:
+        """Shuts down the speech server."""
+
+        server = self._get_server()
+        if server is None:
+            return
+
+        server.shutdownActiveServers()
+        speech.deprecated_clear_server()
+
+    def refresh_speech(self) -> None:
+        """Shuts down and re-initializes speech."""
+
+        self.shutdown_speech()
+        self.start_speech()
+
+    def decrease_rate(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Decreases the speech rate"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.decreaseSpeechRate()
         return True
 
-    def increase_rate(self, script, event=None):
+    def increase_rate(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Increases the speech rate"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.increaseSpeechRate()
         return True
 
-    def decrease_pitch(self, script, event=None):
+    def decrease_pitch(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Decreases the speech pitch"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.decreaseSpeechPitch()
         return True
 
-    def increase_pitch(self, script, event=None):
+    def increase_pitch(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Increase the speech pitch"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.increaseSpeechPitch()
         return True
 
-    def decrease_volume(self, script, event=None):
+    def decrease_volume(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Decreases the speech volume"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.decreaseSpeechVolume()
         return True
 
-    def increase_volume(self, script, event=None):
+    def increase_volume(
+        self, _script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Increases the speech volume"""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.increaseSpeechVolume()
         return True
 
-    def update_capitalization_style(self):
+    def update_capitalization_style(self) -> bool:
         """Updates the capitalization style based on the value in settings."""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.updateCapitalizationStyle()
         return True
 
-    def update_punctuation_level(self):
+    def update_punctuation_level(self) -> bool:
         """Updates the punctuation level based on the value in settings."""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         server.updatePunctuationLevel()
         return True
 
-    def cycle_synthesizer(self, script, event=None):
+    def update_synthesizer(self, server_id: Optional[str] = "") -> None:
+        """Updates the synthesizer to the specified id or value from settings."""
+
+        server = self._get_server()
+        if server is None:
+            msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return
+
+        active_id = server.getOutputModule()
+        info = settings.speechServerInfo or ["", ""]
+        if not server_id and len(info) == 2:
+            server_id = info[1]
+
+        if server_id and server_id != active_id:
+            msg = (
+                f"SPEECH AND VERBOSITY MANAGER: Updating synthesizer from {active_id} "
+                f"to {server_id}."
+            )
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            server.setOutputModule(server_id)
+
+    def cycle_synthesizer(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Cycle through the speech-dispatcher's available output modules."""
 
         server = self._get_server()
         if server is None:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get speech server."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         available = server.list_output_modules()
         if not available:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get output modules."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         current = server.getOutputModule()
         if not current:
             msg = "SPEECH AND VERBOSITY MANAGER: Cannot get current output module."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            debug.print_message(debug.LEVEL_INFO, msg, True)
             return True
 
         index = available.index(current) + 1
@@ -396,7 +507,9 @@ class SpeechAndVerbosityManager:
         script.presentMessage(available[index])
         return True
 
-    def cycle_capitalization_style(self, script, event=None):
+    def cycle_capitalization_style(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Cycle through the speech-dispatcher capitalization styles."""
 
         manager = settings_manager.get_manager()
@@ -419,7 +532,9 @@ class SpeechAndVerbosityManager:
         self.update_capitalization_style()
         return True
 
-    def cycle_punctuation_level(self, script, event=None):
+    def cycle_punctuation_level(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Cycle through the punctuation levels for speech."""
 
         manager = settings_manager.get_manager()
@@ -446,7 +561,9 @@ class SpeechAndVerbosityManager:
         self.update_punctuation_level()
         return True
 
-    def cycle_key_echo(self, script, event=None):
+    def cycle_key_echo(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Cycle through the key echo levels."""
 
         manager = settings_manager.get_manager()
@@ -486,7 +603,9 @@ class SpeechAndVerbosityManager:
         script.presentMessage(full, brief)
         return True
 
-    def change_number_style(self, script, event=None):
+    def change_number_style(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Changes spoken number style between digits and words."""
 
         manager = settings_manager.get_manager()
@@ -502,7 +621,9 @@ class SpeechAndVerbosityManager:
         script.presentMessage(full, brief)
         return True
 
-    def toggle_speech(self, script, event=None):
+    def toggle_speech(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Toggles speech."""
 
         manager = settings_manager.get_manager()
@@ -519,7 +640,9 @@ class SpeechAndVerbosityManager:
             manager.set_setting('silenceSpeech', True)
         return True
 
-    def toggle_verbosity(self, script, event=None):
+    def toggle_verbosity(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Toggles speech verbosity level between verbose and brief."""
 
         manager = settings_manager.get_manager()
@@ -532,7 +655,9 @@ class SpeechAndVerbosityManager:
             manager.set_setting('speechVerbosityLevel', settings.VERBOSITY_LEVEL_BRIEF)
         return True
 
-    def toggle_indentation_and_justification(self, script, event=None):
+    def toggle_indentation_and_justification(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Toggles the speaking of indentation and justification."""
 
         manager = settings_manager.get_manager()
@@ -547,7 +672,9 @@ class SpeechAndVerbosityManager:
         script.presentMessage(full, brief)
         return True
 
-    def toggle_table_cell_reading_mode(self, script, event=None):
+    def toggle_table_cell_reading_mode(
+        self, script: default.Script, _event: Optional[input_event.InputEvent] = None
+    ) -> bool:
         """Toggles between speak cell and speak row."""
 
         table = AXTable.get_table(focus_manager.get_manager().get_locus_of_focus())
@@ -575,7 +702,21 @@ class SpeechAndVerbosityManager:
         return True
 
     @staticmethod
-    def adjust_for_links(obj, line, start_offset):
+    def adjust_for_digits(obj: Atspi.Accessible, string: str) -> str:
+        """Adjusts string to present numbers as digits."""
+
+        def _convert(word):
+            if word.isnumeric():
+                word = " ".join(list(word))
+            return word
+
+        if not (settings.speakNumbersAsDigits or AXUtilities.is_text_input_telephone(obj)):
+            return string
+
+        return " ".join(map(_convert, string.split()))
+
+    @staticmethod
+    def adjust_for_links(obj: Atspi.Accessible, line: str, start_offset: int) -> str:
         """Adjust line to include the word "link" after any hypertext links."""
 
         end_offset = start_offset + len(line)
@@ -584,14 +725,15 @@ class SpeechAndVerbosityManager:
         offsets = sorted([offset - start_offset for offset in offsets], reverse=True)
         tokens = list(line)
         for o in offsets:
-            string = f" {messages.LINK}"
-            if o < len(tokens) and tokens[o].isalnum():
-                string += " "
-            tokens[o:o] = string
+            if 0 <= o <= len(tokens):
+                text = f" {messages.LINK}"
+                if o < len(tokens) and tokens[o].isalnum():
+                    text += " "
+                tokens[o:o] = text
         return "".join(tokens)
 
     @staticmethod
-    def adjust_for_repeats(string):
+    def adjust_for_repeats(string: str) -> str:
         """Adjust line to include a description of repeated symbols."""
 
         def replacement(match):
@@ -607,8 +749,9 @@ class SpeechAndVerbosityManager:
         pattern = re.compile(r"([^a-zA-Z0-9\s])\1{" + str(settings.repeatCharacterLimit - 1) + ",}")
         return re.sub(pattern, replacement, string)
 
-_manager = SpeechAndVerbosityManager()
-def get_manager():
+_manager: SpeechAndVerbosityManager = SpeechAndVerbosityManager()
+
+def get_manager() -> SpeechAndVerbosityManager:
     """Returns the Speech and Verbosity Manager"""
 
     return _manager
